﻿/**
 * @fileoverview オブジェクトを選択していたら規定サイズに縮小し、ロックレイヤーにする。テキストを選択していたら指定フォント・Q数にして、行単位にバラす。アミ指定があるときはアミを下に置く
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
*/
/* 呼び出しの繰り返しを考えるとSPAiと併用したほうがよい。http://tama-san.com/spai/  */

/*
    Copyright:
    2017 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2017 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/
#include "./libCommon.jsx"

/**
 * 選択コンテンツを指定サイズにリサイズし、アートボードをフィットして新レイヤーを作成する
 * @param {Document} document ドキュメントオブジェクト
 * @param {PathItem} obj コンテンツオブジェクト
 * @param {Number} maxwidth 最大幅
 * @type Nothing
*/
function resizePNG(document, obj, maxwidth) {
  maxwidth = mmtopt(maxwidth);
  if (obj.width > maxwidth) {
    var scale = maxwidth / obj.width * 100;
    obj.resize(scale, scale);
    obj.top = 0;
    obj.left = 0;
  }
  document.fitArtboardToSelectedArt(0);
  document.activeLayer.locked = true;
  var newlayer = document.layers.add();
  newlayer.move(document.activeLayer, ElementPlacement.PLACEBEFORE);
}

/**
 * テキストの書体・Q数を設定して行にバラし、網をかける
 * @param {Document} document ドキュメントオブジェクト
 * @param {PathItem} obj コンテンツオブジェクト
 * @param {String} font フォント名
 * @param {Number} q Q数
 * @param {boolean} ami アミ有無
 * @param {Number[]} box アミの左・上・右・下のマージン
 * @param {Number[]} fillcolor アミのC・M・Y・K値
 * @param {boolean} group 文字列とアミのグループ化有無
 * @type Nothing
*/
function splitTexts(document, obj, font, q, ami, box, fillcolor, group) {
  var fontobj = null;
  for (var i = 0; i < app.textFonts.length; i++) {
    if (app.textFonts[i].name.match(/ATC/)) {
      key = "(合成)" + app.textFonts[i].family;
    } else {
      key = app.textFonts[i].family + " " + app.textFonts[i].style;
    }
    if (key == font) {
      fontobj = app.textFonts[i];
      break;
    }
  }

  if (fontobj || q) {
    for (var i = 0; i < obj.characters.length; i++) {
       if (fontobj) obj.characters[i].textFont = fontobj;
       if (q) obj.characters[i].size = qtopt(q);
    }
  }
  splitFrame(document, obj, ami, box, fillcolor, group);
}

/**
 * テキストを行にバラす実体
 * @param {Document} document ドキュメントオブジェクト
 * @param {TextFrame} tf テキストフレーム
 * @param {boolean} ami アミ有無
 * @param {Number[]} box アミの左・上・右・下のマージン
 * @param {Number[]} fillcolor アミのC・M・Y・K値
 * @param {boolean} group 文字列とアミのグループ化有無
 * @type Nothing
*/
function splitFrame(document, tf, ami, box, fillcolor, group) {
  var txPos = tf.position;
  var posX = txPos[0];
  var posY = txPos[1];
  var preLeading = 0;
  for (var j = 0; j < tf.paragraphs.length; j++) {
    var s;
    try {
      s = tf.paragraphs[j].contents;
    } catch(e) {
      s = "";
    }
    var pleading;
    try {
      pleading = tf.paragraphs[j - 1].leading;
    } catch (e) {
      pleading = preLeading;
    }
    posY = posY - pleading;
    if (s != "") {
      var newtf = tf.duplicate();
      newtf.position = [posX, posY];
      newtf.contents = "";
      tf.paragraphs[j].duplicate(newtf);
    }
    preLeading = pleading;
    if (ami == true) setAmi(document, newtf, box, fillcolor, group);
  }
  tf.remove();
}

/**
 * アミをかける実体
 * @param {Document} document ドキュメントオブジェクト
 * @param {TextFrame} tf テキストフレーム
 * @param {Number[]} box アミの左・上・右・下のマージン
 * @param {Number[]} fillcolor アミのC・M・Y・K値
 * @param {boolean} group 文字列とアミのグループ化有無
 * @type Nothing
*/
function setAmi(document, tf, box, fillcolor, group) {
  var left = tf.left - mmtopt(box[0]);
  var width = tf.width + mmtopt(box[0]) + mmtopt(box[2]);
  var top = tf.top + mmtopt(box[1]);
  var height = tf.height + mmtopt(box[1]) + mmtopt(box[3]);
  var rect = document.activeLayer.pathItems.rectangle(top, left, width, height);
  rect.filled = true;
  rect.stroked = false;
  var color = new CMYKColor();
  color.cyan = fillcolor[0];
  color.magenta = fillcolor[1];
  color.yellow = fillcolor[2];
  color.black = fillcolor[3];
  rect.fillColor = color;
  rect.move(tf, ElementPlacement.PLACEAFTER);
  if (group == true) {
    var grp = document.groupItems.add();
    tf.move(grp, ElementPlacement.PLACEATBEGINNING);
    rect.move(grp, ElementPlacement.PLACEATEND);
  }
}

var myDocument = app.activeDocument;
var fObj = getHwjStateFileObj("AI-dialogFigureStringSupport");
var config = {
  maxwidth: 110,
  font1: "A-OTF 新ゴ Pro M",
  q1: 10,
  ami1: false,
  fillcolor1: "0,0,0,0",
  box1: "0.5,0.5,0.5,0.5",
  group1: false,
  font2: "A-OTF 新ゴ Pro R",
  q2: 9,
  ami2: true,
  fillcolor2: "0,0,0,10",
  box2:  "0.5,0.5,0.5,0.5",
  group2: true
};
var array = readHwjStateFile(fObj);
if (array != null) {
  for (var i = 0; i < array.length; i++) {
    var sa = array[i].split("=", 2);
    if (sa[1] == "true") sa[1] = true;
    if (sa[1] == "false") sa[1] = false;
    config[sa[0]] = sa[1];
  }
}
var box1 = config["box1"].split(",");
var box2 = config["box2"].split(",");
var fillcolor1 = config["fillcolor1"].split(",");
var fillcolor2 = config["fillcolor2"].split(",");

if (myDocument.selection[0] instanceof TextFrame) {
  var type = confirm("文字定義1 [はい] / 文字定義2 [いいえ]");
  if (type == true) {
    splitTexts(myDocument, myDocument.selection[0], config["font1"], config["q1"], config["ami1"], box1, fillcolor1, config["group1"]);
  } else {
    splitTexts(myDocument, myDocument.selection[0], config["font2"], config["q2"], config["ami2"], box2, fillcolor2, config["group2"]);
  }
} else if (myDocument.selection[0] instanceof RasterItem) {
  resizePNG(myDocument, myDocument.selection[0], config["maxwidth"]);
} else {
  alert("縮小対象の画像、あるいは乗せる文字テキストフレームを選択してください。");
}
